%* lsaf_addmember                                                                        *;
%*                                                                                       *;
%* Adds a user or a group as a member of the specified context.                          *;
%*                                                                                       *;
%* macrotype LSAF                                                                        *;
%* since 1.2                                                                             *;
%* exposure external                                                                     *;
%*                                                                                       *;
%* History                                                                               *;
%* 2013-08-30  updates for standardization and coding conventions                        *;
%* 2016-08-29  rebrand as LSAF                                                           *;
%* 2019-01-15  updated for LSAF 5.1 functionality                                        *;
%* 2020-12-04  updates for service refactor                                              *;
%* 2021-08-09  Remove header documentation                                               *;
%* 2021-09-20  Remove parameter defaults (DE16775)                                       *;
%* 2021-10-20  principal refactor                                                        *;
%*                                                                                       *;

%macro lsaf_addmember(
  lsaf_path=,
  lsaf_member=,
  lsaf_type=,
  lsaf_group_context=
  ) /des='Add the member to a context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/security/membership/MembershipService");
	declare javaobj member("com/sas/lsaf/macro/security/SasPrincipalDescriptor", 
                           "%bquote(&lsaf_type)", "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)");
    declare javaobj results("com/sas/lsaf/macro/common/result/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("addMember", "%bquote(&lsaf_path)", member, results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      if upcase("&lsaf_type")="GROUP"
        then logMsg=" The &lsaf_type &lsaf_group_context &lsaf_member has been added to &lsaf_path.." ;
        else logMsg=" The &lsaf_type &lsaf_member has been added to &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if upcase("&lsaf_type")="GROUP"
        then logMsg=" Cannot add &lsaf_type &lsaf_group_context &lsaf_member to &lsaf_path.." ;
        else logMsg=" Cannot add &lsaf_type &lsaf_member to &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " &_LSAF_PRODUCT_NAME_ macro " %sysfunc(lowcase("&SYSMACRONAME")) ;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_addmember;
